@@ -24,6 +24,7 @@
 
 #include <glib/gi18n-lib.h>
 
+#include "server/e-ews-connection-utils.h"
 #include "server/e-source-ews-folder.h"
 
 #define E_EWS_BACKEND_GET_PRIVATE(obj) \
@@ -43,6 +44,7 @@ struct _EEwsBackendPrivate {
 	gchar *sync_state;
 	GMutex sync_state_lock;
 
+	ENamedParameters *credentials;
 	EEwsConnection *connection;
 	GMutex connection_lock;
 
@@ -579,6 +581,8 @@ ews_backend_finalize (GObject *object)
 
 	g_mutex_clear (&priv->connection_lock);
 
+	e_named_parameters_free (priv->credentials);
+
 	/* Chain up to parent's finalize() method. */
 	G_OBJECT_CLASS (e_ews_backend_parent_class)->finalize (object);
 }
@@ -684,10 +688,21 @@ ews_backend_populate (ECollectionBackend *backend)
 
 	ews_backend_add_gal_source (ews_backend);
 
-	if (e_backend_get_online (E_BACKEND (backend)))
-		e_ews_backend_sync_folders (ews_backend, NULL, ews_backend_folders_synced_cb, NULL);
-	else
+	if (e_backend_get_online (E_BACKEND (backend))) {
+		CamelEwsSettings *ews_settings;
+
+		ews_settings = ews_backend_get_settings (ews_backend);
+
+		if (e_ews_connection_utils_get_without_password (ews_settings)) {
+			e_backend_schedule_authenticate (E_BACKEND (backend), NULL);
+		} else {
+			e_backend_credentials_required_sync (E_BACKEND (backend),
+				E_SOURCE_CREDENTIALS_REASON_REQUIRED, NULL, 0, NULL,
+				NULL, NULL);
+		}
+	} else {
 		ews_backend_claim_old_resources (backend);
+	}
 }
 
 static gchar *
@@ -815,8 +830,7 @@ ews_backend_create_resource_sync (ECollectionBackend *backend,
 	}
 
 	if (!success) {
-		connection = e_ews_backend_ref_connection_sync (
-			E_EWS_BACKEND (backend), cancellable, error);
+		connection = e_ews_backend_ref_connection_sync (E_EWS_BACKEND (backend), NULL, cancellable, error);
 		if (connection == NULL)
 			return FALSE;
 
@@ -923,8 +937,7 @@ ews_backend_delete_resource_sync (ECollectionBackend *backend,
 	const gchar *extension_name;
 	gboolean success = FALSE;
 
-	connection = e_ews_backend_ref_connection_sync (
-		E_EWS_BACKEND (backend), cancellable, error);
+	connection = e_ews_backend_ref_connection_sync (E_EWS_BACKEND (backend), NULL, cancellable, error);
 	if (connection == NULL)
 		return FALSE;
 
@@ -1004,6 +1017,44 @@ ews_backend_get_destination_address (EBackend *backend,
 	return result;
 }
 
+static ESourceAuthenticationResult
+ews_backend_authenticate_sync (EBackend *backend,
+			       const ENamedParameters *credentials,
+			       gchar **out_certificate_pem,
+			       GTlsCertificateFlags *out_certificate_errors,
+			       GCancellable *cancellable,
+			       GError **error)
+{
+	EEwsBackend *ews_backend;
+	EEwsConnection *connection;
+	CamelEwsSettings *ews_settings;
+	ESourceAuthenticationResult result = E_SOURCE_AUTHENTICATION_ERROR;
+	gchar *hosturl;
+
+	g_return_val_if_fail (E_IS_EWS_BACKEND (backend), E_SOURCE_AUTHENTICATION_ERROR);
+
+	ews_backend = E_EWS_BACKEND (backend);
+	ews_settings = ews_backend_get_settings (ews_backend);
+	g_return_val_if_fail (ews_settings != NULL, E_SOURCE_AUTHENTICATION_ERROR);
+
+	g_mutex_lock (&ews_backend->priv->connection_lock);
+	g_clear_object (&ews_backend->priv->connection);
+	e_named_parameters_free (ews_backend->priv->credentials);
+	ews_backend->priv->credentials = e_named_parameters_new_clone (credentials);
+	g_mutex_unlock (&ews_backend->priv->connection_lock);
+
+	connection = e_ews_backend_ref_connection_sync (ews_backend, &result, cancellable, error);
+	g_clear_object (&connection);
+
+	if (result == E_SOURCE_AUTHENTICATION_ACCEPTED) {
+		e_collection_backend_authenticate_children (E_COLLECTION_BACKEND (backend), credentials);
+
+		e_ews_backend_sync_folders (ews_backend, NULL, ews_backend_folders_synced_cb, NULL);
+	}
+
+	return result;
+}
+
 static void
 e_ews_backend_class_init (EEwsBackendClass *class)
 {
@@ -1028,6 +1079,7 @@ e_ews_backend_class_init (EEwsBackendClass *class)
 
 	backend_class = E_BACKEND_CLASS (class);
 	backend_class->get_destination_address = ews_backend_get_destination_address;
+	backend_class->authenticate_sync = ews_backend_authenticate_sync;
 
 	/* This generates an ESourceCamel subtype for CamelEwsSettings. */
 	e_source_camel_generate_subtype ("ews", CAMEL_TYPE_EWS_SETTINGS);
@@ -1067,8 +1119,7 @@ ews_backend_ref_connection_thread (GSimpleAsyncResult *simple,
 	EEwsConnection *connection;
 	GError *error = NULL;
 
-	connection = e_ews_backend_ref_connection_sync (
-		E_EWS_BACKEND (object), cancellable, &error);
+	connection = e_ews_backend_ref_connection_sync (E_EWS_BACKEND (object), NULL, cancellable, &error);
 
 	/* Sanity check. */
 	g_return_if_fail (
@@ -1085,10 +1136,12 @@ ews_backend_ref_connection_thread (GSimpleAsyncResult *simple,
 
 EEwsConnection *
 e_ews_backend_ref_connection_sync (EEwsBackend *backend,
+				   ESourceAuthenticationResult *result,
                                    GCancellable *cancellable,
                                    GError **error)
 {
 	EEwsConnection *connection = NULL;
+	ESourceAuthenticationResult local_result;
 	CamelEwsSettings *settings;
 	gchar *hosturl;
 	gboolean success;
@@ -1102,7 +1155,7 @@ e_ews_backend_ref_connection_sync (EEwsBackend *backend,
 
 	/* If we already have an authenticated
 	 * connection object, just return that. */
-	if (connection != NULL)
+	if (connection != NULL || !backend->priv->credentials)
 		return connection;
 
 	settings = ews_backend_get_settings (backend);
@@ -1115,10 +1168,11 @@ e_ews_backend_ref_connection_sync (EEwsBackend *backend,
 		connection, "proxy-resolver",
 		G_BINDING_SYNC_CREATE);
 
-	success = e_backend_authenticate_sync (
-		E_BACKEND (backend),
-		E_SOURCE_AUTHENTICATOR (connection),
-		cancellable, error);
+	local_result = e_ews_connection_try_credentials_sync (connection, backend->priv->credentials, cancellable, error);
+	if (result)
+		*result = local_result;
+
+	success = local_result == E_SOURCE_AUTHENTICATION_ACCEPTED;
 
 	if (success) {
 		g_mutex_lock (&backend->priv->connection_lock);
@@ -1255,8 +1309,7 @@ e_ews_backend_sync_folders_sync (EEwsBackend *backend,
 		return TRUE;
 	}
 
-	connection = e_ews_backend_ref_connection_sync (
-		backend, cancellable, error);
+	connection = e_ews_backend_ref_connection_sync (backend, NULL, cancellable, error);
 
 	if (connection == NULL) {
 		backend->priv->need_update_folders = TRUE;
