@@ -20,8 +20,6 @@
 #include <config.h>
 #endif
 
-#include "e-mail-config-ews-ooo-page.h"
-
 #include <string.h>
 #include <unistd.h>
 
@@ -35,6 +33,10 @@
 #include "server/e-ews-connection-utils.h"
 #include "server/e-ews-oof-settings.h"
 
+#include "e-ews-config-utils.h"
+
+#include "e-mail-config-ews-ooo-page.h"
+
 #define E_MAIL_CONFIG_EWS_OOO_PAGE_GET_PRIVATE(obj) \
 	(G_TYPE_INSTANCE_GET_PRIVATE \
 	((obj), E_TYPE_MAIL_CONFIG_EWS_OOO_PAGE, EMailConfigEwsOooPagePrivate))
@@ -70,6 +72,8 @@ struct _EMailConfigEwsOooPagePrivate {
 struct _AsyncContext {
 	EMailConfigEwsOooPage *page;
 	EActivity *activity;
+	ESource *source;
+	GObject *settings;
 };
 
 enum {
@@ -83,8 +87,6 @@ enum {
 /* Forward Declarations */
 static void	e_mail_config_ews_ooo_page_interface_init
 				(EMailConfigPageInterface *iface);
-static void	e_mail_config_ews_ooo_page_authenticator_init
-				(ESourceAuthenticatorInterface *iface);
 
 G_DEFINE_DYNAMIC_TYPE_EXTENDED (
 	EMailConfigEwsOooPage,
@@ -93,19 +95,23 @@ G_DEFINE_DYNAMIC_TYPE_EXTENDED (
 	0,
 	G_IMPLEMENT_INTERFACE_DYNAMIC (
 		E_TYPE_MAIL_CONFIG_PAGE,
-		e_mail_config_ews_ooo_page_interface_init)
-	G_IMPLEMENT_INTERFACE_DYNAMIC (
-		E_TYPE_SOURCE_AUTHENTICATOR,
-		e_mail_config_ews_ooo_page_authenticator_init))
+		e_mail_config_ews_ooo_page_interface_init))
 
 static void
-async_context_free (AsyncContext *async_context)
+async_context_free (gpointer ptr)
 {
-	if (async_context->page != NULL)
-		g_object_unref (async_context->page);
+	AsyncContext *async_context = ptr;
 
-	if (async_context->activity != NULL)
-		g_object_unref (async_context->activity);
+	if (!async_context)
+		return;
+
+	if (async_context->settings)
+		g_object_thaw_notify (async_context->settings);
+
+	g_clear_object (&async_context->page);
+	g_clear_object (&async_context->activity);
+	g_clear_object (&async_context->source);
+	g_clear_object (&async_context->settings);
 
 	g_slice_free (AsyncContext, async_context);
 }
@@ -213,48 +219,6 @@ mail_config_ews_ooo_page_display_settings (EMailConfigEwsOooPage *page,
 		e_ews_oof_settings_get_external_reply (oof_settings), -1);
 }
 
-static void
-mail_config_ews_ooo_page_refresh_cb (GObject *source_object,
-                                     GAsyncResult *result,
-                                     gpointer user_data)
-{
-	ESourceRegistry *registry;
-	AsyncContext *async_context;
-	EAlertSink *alert_sink;
-	GError *error = NULL;
-
-	registry = E_SOURCE_REGISTRY (source_object);
-	async_context = (AsyncContext *) user_data;
-
-	alert_sink = e_activity_get_alert_sink (async_context->activity);
-
-	e_source_registry_authenticate_finish (registry, result, &error);
-
-	if (e_activity_handle_cancellation (async_context->activity, error)) {
-		g_error_free (error);
-
-	} else if (error != NULL) {
-		e_alert_submit (
-			alert_sink,
-			"ews:query-ooo-error",
-			error->message, NULL);
-		g_error_free (error);
-
-	} else {
-		EMailConfigEwsOooPage *page = async_context->page;
-
-		g_mutex_lock (&page->priv->oof_settings_lock);
-
-		if (page->priv->oof_settings != NULL)
-			mail_config_ews_ooo_page_display_settings (
-				page, page->priv->oof_settings);
-
-		g_mutex_unlock (&page->priv->oof_settings_lock);
-	}
-
-	async_context_free (async_context);
-}
-
 static void
 mail_config_ews_ooo_page_set_account_source (EMailConfigEwsOooPage *page,
                                              ESource *account_source)
@@ -448,6 +412,7 @@ mail_config_ews_ooo_page_constructed (GObject *object)
 	text = _("The messages specified below will be automatically sent to "
 		 "each internal and external person who sends a mail to you.");
 	widget = gtk_label_new (text);
+	gtk_label_set_max_width_chars (GTK_LABEL (widget), 80);
 	gtk_label_set_line_wrap (GTK_LABEL (widget), TRUE);
 	gtk_misc_set_alignment (GTK_MISC (widget), 0.0, 0.5);
 	gtk_box_pack_start (GTK_BOX (container), widget, FALSE, FALSE, 0);
@@ -793,55 +758,26 @@ mail_config_ews_ooo_page_submit_finish (EMailConfigPage *page,
 	return !g_simple_async_result_propagate_error (simple, error);
 }
 
-static gboolean
-mail_config_ews_ooo_page_get_without_password (ESourceAuthenticator *auth)
-{
-	EMailConfigEwsOooPage *page;
-	CamelSettings *settings;
-	CamelEwsSettings *ews_settings;
-
-	page = E_MAIL_CONFIG_EWS_OOO_PAGE (auth);
-	settings = mail_config_ews_ooo_page_get_settings (page);
-	ews_settings = CAMEL_EWS_SETTINGS (settings);
-
-	return e_ews_connection_utils_get_without_password (ews_settings);
-}
-
 static ESourceAuthenticationResult
-mail_config_ews_ooo_page_try_password_sync (ESourceAuthenticator *auth,
-                                            const GString *password,
-                                            GCancellable *cancellable,
-                                            GError **error)
+mail_config_ews_ooo_page_try_credentials_sync (EEwsConnection *connection,
+					       const ENamedParameters *credentials,
+					       gpointer user_data,
+					       GCancellable *cancellable,
+					       GError **error)
 {
+	AsyncContext *async_context = user_data;
 	EMailConfigEwsOooPage *page;
-	CamelSettings *settings;
-	CamelEwsSettings *ews_settings;
 	ESourceAuthenticationResult result;
-	EEwsConnection *connection;
 	EEwsOofSettings *oof_settings;
-	const gchar *hosturl;
 	const gchar *mailbox;
 	GError *local_error = NULL;
 
-	if (g_cancellable_set_error_if_cancelled (cancellable, error))
-		return E_SOURCE_AUTHENTICATION_ERROR;
+	page = async_context->page;
 
-	page = E_MAIL_CONFIG_EWS_OOO_PAGE (auth);
 	mailbox = mail_config_ews_ooo_page_get_mailbox (page);
-	settings = mail_config_ews_ooo_page_get_settings (page);
-
-	ews_settings = CAMEL_EWS_SETTINGS (settings);
-	hosturl = camel_ews_settings_get_hosturl (ews_settings);
-
-	connection = e_ews_connection_new (hosturl, ews_settings);
-	e_ews_connection_set_password (connection, password->str);
-
 	e_ews_connection_set_mailbox (connection, mailbox);
 
-	oof_settings = e_ews_oof_settings_new_sync (
-		connection, cancellable, &local_error);
-
-	g_object_unref (connection);
+	oof_settings = e_ews_oof_settings_new_sync (connection, cancellable, &local_error);
 
 	if (oof_settings != NULL) {
 		result = E_SOURCE_AUTHENTICATION_ACCEPTED;
@@ -867,6 +803,68 @@ mail_config_ews_ooo_page_try_password_sync (ESourceAuthenticator *auth,
 	return result;
 }
 
+static void
+mail_config_ews_ooo_page_refresh_thread_cb (GObject *with_object,
+					    gpointer user_data,
+					    GCancellable *cancellable,
+					    GError **perror)
+{
+	AsyncContext *async_context = user_data;
+	CamelEwsSettings *ews_settings;
+	EEwsConnection *connection;
+
+	if (g_cancellable_set_error_if_cancelled (cancellable, perror))
+		return;
+
+	ews_settings = CAMEL_EWS_SETTINGS (async_context->settings);
+	connection = e_ews_config_utils_open_connection_for (async_context->source, ews_settings, NULL,
+		mail_config_ews_ooo_page_try_credentials_sync, async_context, cancellable, perror);
+
+	g_clear_object (&connection);
+}
+
+static void
+mail_config_ews_ooo_page_refresh_idle_cb (GObject *with_object,
+					  gpointer user_data,
+					  GCancellable *cancellable,
+					  GError **perror)
+{
+	AsyncContext *async_context;
+	EAlertSink *alert_sink;
+	GError *error = NULL;
+
+	async_context = (AsyncContext *) user_data;
+
+	if (perror) {
+		error = *perror;
+		*perror = NULL;
+	}
+
+	alert_sink = e_activity_get_alert_sink (async_context->activity);
+
+	if (e_activity_handle_cancellation (async_context->activity, error)) {
+		g_error_free (error);
+
+	} else if (error != NULL) {
+		e_alert_submit (
+			alert_sink,
+			"ews:query-ooo-error",
+			error->message, NULL);
+		g_error_free (error);
+
+	} else {
+		EMailConfigEwsOooPage *page = async_context->page;
+
+		g_mutex_lock (&page->priv->oof_settings_lock);
+
+		if (page->priv->oof_settings != NULL)
+			mail_config_ews_ooo_page_display_settings (
+				page, page->priv->oof_settings);
+
+		g_mutex_unlock (&page->priv->oof_settings_lock);
+	}
+}
+
 static void
 e_mail_config_ews_ooo_page_class_init (EMailConfigEwsOooPageClass *class)
 {
@@ -936,15 +934,6 @@ e_mail_config_ews_ooo_page_interface_init (EMailConfigPageInterface *iface)
 	iface->submit_finish = mail_config_ews_ooo_page_submit_finish;
 }
 
-static void
-e_mail_config_ews_ooo_page_authenticator_init (ESourceAuthenticatorInterface *iface)
-{
-	iface->get_without_password =
-		mail_config_ews_ooo_page_get_without_password;
-	iface->try_password_sync =
-		mail_config_ews_ooo_page_try_password_sync;
-}
-
 static void
 e_mail_config_ews_ooo_page_class_finalize (EMailConfigEwsOooPageClass *class)
 {
@@ -990,18 +979,15 @@ e_mail_config_ews_ooo_page_new (ESourceRegistry *registry,
 void
 e_mail_config_ews_ooo_page_refresh (EMailConfigEwsOooPage *page)
 {
-	ESourceAuthenticator *authenticator;
-	ESourceRegistry *registry;
 	ESource *source;
 	EActivity *activity;
 	GCancellable *cancellable;
+	CamelSettings *settings;
 	AsyncContext *async_context;
 
 	g_return_if_fail (E_IS_MAIL_CONFIG_EWS_OOO_PAGE (page));
 
-	registry = e_mail_config_ews_ooo_page_get_registry (page);
 	source = e_mail_config_ews_ooo_page_get_collection_source (page);
-	authenticator = E_SOURCE_AUTHENTICATOR (page);
 
 	if (page->priv->refresh_cancellable) {
 		g_cancellable_cancel (page->priv->refresh_cancellable);
@@ -1016,13 +1002,22 @@ e_mail_config_ews_ooo_page_refresh (EMailConfigEwsOooPage *page)
 	e_activity_set_text (
 		activity, _("Retrieving \"Out of Office\" settings"));
 
+	settings = mail_config_ews_ooo_page_get_settings (page);
+
 	async_context = g_slice_new0 (AsyncContext);
 	async_context->page = g_object_ref (page);
 	async_context->activity = activity;  /* takes ownership */
+	async_context->source = g_object_ref (source);
+	async_context->settings = g_object_ref (settings);
 
-	e_source_registry_authenticate (
-		registry, source, authenticator, cancellable,
-		mail_config_ews_ooo_page_refresh_cb, async_context);
+	/* Property changes can cause update of the UI, but this runs in a thread,
+	   thus freeze the notify till be back in UI thread */
+	g_object_freeze_notify (async_context->settings);
+
+	e_ews_config_utils_run_in_thread (G_OBJECT (page),
+		mail_config_ews_ooo_page_refresh_thread_cb,
+		mail_config_ews_ooo_page_refresh_idle_cb,
+		async_context, async_context_free, cancellable);
 }
 
 ESourceRegistry *
